<?php

namespace Litecms\Addons\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Addons\Http\Requests\AddonRequest;
use Litecms\Addons\Interfaces\AddonRepositoryInterface;
use Litecms\Addons\Models\Addon;

/**
 * Resource controller class for addon.
 */
class AddonResourceController extends BaseController
{

    /**
     * Initialize addon resource controller.
     *
     * @param type AddonRepositoryInterface $addon
     *
     * @return null
     */
    public function __construct(AddonRepositoryInterface $addon)
    {
        parent::__construct();
        $this->repository = $addon;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Addons\Repositories\Criteria\AddonResourceCriteria::class);
    }

    /**
     * Display a list of addon.
     *
     * @return Response
     */
    public function index(AddonRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Addons\Repositories\Presenter\AddonPresenter::class)
                ->$function();
        }

        $addons = $this->repository->paginate();

        return $this->response->title(trans('addons::addon.names'))
            ->view('addons::addon.index', true)
            ->data(compact('addons'))
            ->output();
    }

    /**
     * Display addon.
     *
     * @param Request $request
     * @param Model   $addon
     *
     * @return Response
     */
    public function show(AddonRequest $request, Addon $addon)
    {

        if ($addon->exists) {
            $view = 'addons::addon.show';
        } else {
            $view = 'addons::addon.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('addons::addon.name'))
            ->data(compact('addon'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new addon.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AddonRequest $request)
    {

        $addon = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('addons::addon.name')) 
            ->view('addons::addon.create', true) 
            ->data(compact('addon'))
            ->output();
    }

    /**
     * Create new addon.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AddonRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $addon                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('addons::addon.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('addons/addon/' . $addon->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/addons/addon'))
                ->redirect();
        }

    }

    /**
     * Show addon for editing.
     *
     * @param Request $request
     * @param Model   $addon
     *
     * @return Response
     */
    public function edit(AddonRequest $request, Addon $addon)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('addons::addon.name'))
            ->view('addons::addon.edit', true)
            ->data(compact('addon'))
            ->output();
    }

    /**
     * Update the addon.
     *
     * @param Request $request
     * @param Model   $addon
     *
     * @return Response
     */
    public function update(AddonRequest $request, Addon $addon)
    {
        try {
            $attributes = $request->all();

            $addon->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('addons::addon.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('addons/addon/' . $addon->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('addons/addon/' . $addon->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the addon.
     *
     * @param Model   $addon
     *
     * @return Response
     */
    public function destroy(AddonRequest $request, Addon $addon)
    {
        try {

            $addon->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('addons::addon.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('addons/addon/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('addons/addon/' . $addon->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple addon.
     *
     * @param Model   $addon
     *
     * @return Response
     */
    public function delete(AddonRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('addons::addon.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('addons/addon'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/addons/addon'))
                ->redirect();
        }

    }

    /**
     * Restore deleted addons.
     *
     * @param Model   $addon
     *
     * @return Response
     */
    public function restore(AddonRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('addons::addon.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/addons/addon'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/addons/addon/'))
                ->redirect();
        }

    }

}
